library httpserver;

import "dart:io";
import "dart:async";

import "typed_pipe_config.dart";

Stream<Connection> start_server({String host : "localhost" , int port : 8888}) =>
  new _HttpServer(host,port).stream;
  
class Connection{
  
  final IsyumiPipe ip;
  final HttpRequest _req;
  
  Connection(this._req,WebSocket ws):ip = getPipe(ws, ws.add);
  
  void cookies(void f(String name,String value)){
    _req.cookies.forEach((cookie) => f(cookie.name,cookie.value));
    
  }
  
}

class _HttpServer{
  
  final String host;
  final int port;
  
  _HttpServer(this.host,this.port){
    HttpServer.bind(host, port).then(onHttpServer);
  }
  
  final StreamController<Connection> sc = new StreamController<Connection>() ;

  Stream<Connection> get stream => sc.stream;

  void onHttpServer(HttpServer server){
    server.listen(onRequest);
  }

  void onRequest(HttpRequest req){
    if(WebSocketTransformer.isUpgradeRequest(req)){
      WebSocketTransformer.upgrade(req).then((ws) => onWebSocket(req,ws));
    }else{
      onHttpRequest(req);
    }
  }
  
  void onHttpRequest(HttpRequest req){
    
    if(req.uri.path == "/"){
      index(req);
      return;
    }
    
    //ここあとでハッキングされないように作り変える。
    put_file(req,req.uri.path);
    
  }
  
  void index(HttpRequest req){
    put_file(req,"/index.html");
    
  }
  
  void put_file(HttpRequest req, String path){

    new File.fromUri(new Uri.file( "../lib/httpserver/www" + path))
      .openRead().listen(
          (data) => response(req,data),
          onDone : () => req.response.close(),
          onError: (e)=> not_found(req),
          cancelOnError: true);
    

  }
  
  void response(HttpRequest req , List<int> data){
    
    if(req.uri.path.endsWith(".dart")){
      addContentType(req,"application/dart");
    }else{
      addContentType(req, "text/html");
    }
    
    req.response.add(data);
    
  }
  
  void addContentType(HttpRequest req,String value){
    req.response.headers.add("content-type", value);
  }
  
  
  
  void not_found(HttpRequest request){
    request.response
      ..statusCode = HttpStatus.NOT_FOUND
      ..write("not found")
      ..close();
  }

  void onWebSocket(HttpRequest req,WebSocket ws){
    sc.add(new Connection(req,ws));
  }
  
}